/*
$Header: d:/cvsroot/tads/TADS2/mac/OSMAC.H,v 1.2 1999/05/17 02:52:18 MJRoberts Exp $
*/

/* Copyright (c) 1992 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  os.h - operating system definitions for Macintosh
Function
  Definitions that vary by operating system
Notes
  None
Modified
  12/26/97 CNebel    - updated for modern compilers.
  06/11/92 MJRoberts     - creation
*/

#ifndef OSMAC_INCLUDED
#define OSMAC_INCLUDED

#ifdef MAC_OS

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>

#define OS_SYSTEM_NAME  "Mac OS"
#define OS_SYSTEM_LDESC  "Mac OS"

/* maximum width of an output line buffer */
#define OS_MAXWIDTH 135

/* far pointer type qualifier (null on most platforms) */
#define osfar_t
#define far

/* maximum theoretical size of malloc argument */
#define OSMALMAX ((size_t)0xffffffff)

/* cast an expression to void */
#define DISCARD

/* display lines on which errors occur */
#define OS_ERRLINE 1

/*
 *   If long cache-manager macros are NOT allowed, define
 *   OS_MCM_NO_MACRO.  This forces certain cache manager operations to be
 *   functions, which results in substantial memory savings.  
 */
#define OS_MCM_NO_MACRO 1

/* likewise for the error handling subsystem */
#define ERR_NO_MACRO 1


/*
 *   If error messages are to be included in the executable, define
 *   ERR_LINK_MESSAGES.  Otherwise, they'll be read from an external
 *   file that is to be opened with oserrop().
 */
#define ERR_LINK_MESSAGES 1

/*
 *   If filename extensions are not normally used in the OS,
 *   define OS_NO_EXTENSIONS.
 */
#define OS_NO_EXTENSIONS 1

/* round a size to worst-case alignment boundary */
#define osrndsz(s) (((s)+1) & ~1)

/* round a pointer to worst-case alignment boundary */
#define osrndpt(p) ((uchar *)((((ulong)(p)) + 1) & ~1))

/* void pointer type */
typedef void dvoid;

/* offset of a member inside a structure */
#ifndef offsetof
#define offsetof(typ, mem) ((uint)&(((typ *)0)->mem))
#endif

/* service macros for osrp2 etc. */
#define osc2u(p, i) ((uint)(((uchar *)(p))[i]))
#define osc2l(p, i) ((ulong)(((uchar *)(p))[i]))

/* read unaligned portable 2-byte value, returning int */
#define osrp2(p) (osc2u(p, 0) + (osc2u(p, 1) << 8))

/* write int to unaligned portable 2-byte value */
#define oswp2(p, i) ((((uchar *)(p))[1] = (i)>>8), (((uchar *)(p))[0] = (i)&255))

/* read unaligned portable 4-byte value, returning long */
#define osrp4(p) \
 (osc2l(p, 0) + (osc2l(p, 1) << 8) + (osc2l(p, 2) << 16) + (osc2l(p, 3) << 24))

/* write long to unaligned portable 4-byte value */
#define oswp4(p, i) \
 ((((uchar *)(p))[0] = (i)), (((uchar *)(p))[1] = (i)>>8),\
  (((uchar *)(p))[2] = (i)>>16, (((uchar *)(p))[3] = (i)>>24)))

/* allocate storage - malloc where supported */
dvoid *osmalloc(size_t siz);

/* free storage allocated with osmalloc */
void osfree(dvoid *block);

/* copy a structure - dst and src are structures, not pointers */
#define OSCPYSTRUCT(dst, src) ((dst) = (src))

/* is a full ANSI compiler */
#define OSANSI

/* maximum length of a filename */
#define OSFNMAX  255

/* normal path separator character */
#define OSPATHCHAR ':'

/* alternate path separator characters */
#define OSPATHALT ":"

/* os file structure */
typedef FILE osfildef;

/* main program exit codes */
#define OSEXSUCC 0                 /* successful completion */
#define OSEXFAIL 1                    /* error occurred */


/* debugging routines for tracking file opens and closes. */
#ifdef __cplusplus
    #if DEBUG_IO
        void track_fileopen(const char *routine, const char *fname, osfildef *fp);
        void track_fileclose(const char *routine, osfildef *fp);
    #else
        #define track_fileopen(routine, fname, fp)  ((void) 0)
        #define track_fileclose(routine, fp) ((void) 0)
    #endif
#endif

/* open text file for reading; returns NULL on error */
#ifdef __cplusplus
inline osfildef *osfoprt(const char *fname, int /*typ*/)
{
    osfildef  *fp = fopen(fname, "r");
    
    track_fileopen("osfoprt", fname, fp);
    return fp;
}
#else
#define osfoprt(fname, typ) fopen(fname, "r")
#endif

/* open text file for writing; returns NULL on error */
osfildef *osfopwt(const char *fname, int typ);

/* open source file for reading */
#ifdef __cplusplus
inline osfildef *osfoprs(const char *fname, int typ)
{
    return osfoprt(fname, typ);
}
#else
#define osfoprs(fname, typ) osfoprt(fname, typ)
#endif

/* open binary file for writing; returns NULL on error */
osfildef *osfopwb(const char *fname, int typ);

/* open binary file for reading; returns NULL on erorr */
#ifdef __cplusplus
inline osfildef *osfoprb(const char *fname, int /*typ*/)
{
    osfildef  *fp = fopen(fname, "rb");
    
    track_fileopen("osfoprb", fname, fp);
    return fp;
}
#else
#define osfoprb(fname, typ) fopen(fname, "rb")
#endif

/* get a line of text from a text file (fgets semantics) */
char *osfgets(char *buf, int len, osfildef *fp);
    /* Need a real function because TADS requires functionality not in the library fgets. */

/* write a line of text to a text file (fputs semantics) */
/* void osfputs(char *buf, osfildef *fp); */
#define osfputs(buf, fp) fputs(buf, fp)

/* open binary file for reading/writing; truncate; returns NULL on error */
osfildef *osfoprwtb(const char *fname, int typ);

/* open binary file for reading/writing, no truncation */
#ifdef __cplusplus
inline osfildef *osfoprwb(const char *fname, os_filetype_t /*typ*/)
{
    osfildef  *fp = fopen(fname, "r+b");

    /* if we failed to open an existing file, try creating a new one */
    if (fp == 0)
        fp = fopen(fname, "w+b");
    
    track_fileopen("osfoprwb", fname, fp);
    return fp;
}
#else
osfildef *osfoprwb(const char *fname, os_filetype_t typ);
#endif

/* write bytes to file; TRUE ==> error */
#ifdef __cplusplus
inline int osfwb(osfildef *fp, const void *buf, size_t bufl)
{
    size_t  ret;
    ret = fwrite(buf, bufl, 1, fp);
    #if DEBUG_IO
    extern void printmem(const void *p, size_t len);
    
    printf("[F: osfwb(fp, ("); printmem(buf, bufl); printf("), %d) --> %d]\n", bufl, ret);
    #endif
    return ret != 1;
}
#else
#define osfwb(fp, buf, bufl) (fwrite(buf, bufl, 1, fp) != 1)
#endif

/* read bytes from file; TRUE ==> error */
/* int osfrb(osfildef *fp, uchar *buf, int bufl); */
#ifdef __cplusplus
inline int osfrb(osfildef *fp, void *buf, size_t bufl)
{
    size_t  ret;
    
    ret = fread(buf, bufl, 1, fp);
    #if DEBUG_IO
    printf("[F: osfrb(fp, buf, %d) --> %d]\n", bufl, ret);
    #endif
    return ret != 1;
}
#else
#define osfrb(fp, buf, bufl) (fread(buf, bufl, 1, fp) != 1)
#endif

/* get position in file */
/* long osfpos(osfildef *fp); */
#define osfpos(fp) ftell(fp)

/* seek position in file; TRUE ==> error */
/* int osfseek(osfildef *fp, long pos, int mode); */
#define osfseek(fp, pos, mode) fseek(fp, pos, mode)
#define OSFSK_SET  SEEK_SET
#define OSFSK_CUR  SEEK_CUR
#define OSFSK_END  SEEK_END

/* close a file */
/* void osfcls(osfildef *fp); */
#ifdef __cplusplus
inline void osfcls(osfildef *fp)
{
    #if DEBUG_IO
    track_fileclose("osfcls", fp);
    #endif
    fclose(fp);
}
#else
#define osfcls(fp) fclose(fp)
#endif

/* delete a file - TRUE if error */
/* int osfdel(char *fname); */
#ifdef __cplusplus
inline int osfdel(const char *fname)
{
    #if DEBUG_IO
    printf("[F: osfdel(%s)]\n", fname);
    #endif
    return remove(fname);
}
#else
#define osfdel(fname) remove(fname)
#endif

/* access a file - 0 if file exists */
int osfacc(const char *fname);

/* open error file, returning file handle, or null if not found */
//osfildef *oserrop(char *arg0);

/*
 *   os_settype(file, type) sets a file to the given type.  This is
 *   required for certain operating systems which like to classify
 *   their files through these type identifiers.  The OSFTxxx constants
 *   are the abstract types we use; the os_settype implementation must
 *   translate to an appropriate OS type.  (This routine is a no-op on
 *   many systems.)
 */
#define OSFTGAME 0      /* a game data file (.gam) */
#define OSFTSAVE 1      /* a saved game (.sav) */
#define OSFTLOG  2      /* a transcript (log) file */
#define OSFTSWAP 3      /* swap file */
#define OSFTDATA 4      /* user data file (used with the TADS fopen() call) */
#define OSFTCMD  5      /* QA command/log file */
#define OSFTERRS 6      /* error message file */
#define OSFTTEXT 7      /* text file - used for source files */
#define OSFTBIN  8      /* binary file of unknown type - resources, etc */
#define OSFTCMAP 9  /* character mapping file */
#define OSFTPREF 10 /* preferences file */

/* get a character from a file */
/* int osfgetc(osfildef *fp); */
#define osfgetc(fp) fgetc(fp)

/* do nothing with OS_LOADDS definitions */
#define OS_LOADDS

/*
 *   Options for this operating system
 */
#define USE_DOSEXT
#define USE_EXPAUSE
#define USE_TIMERAND

/* update progress display with linfdef info, if appropriate */
void os_progress(struct linfdef *linf);

/*
 *   os_squote and os_dquote return TRUE if a character is
 *   a single quote or double quote character, respectively.
 *   This allows character sets containing extended quote
 *   characters to be used.
 */
//#define os_squote(c) ((c) == '\'' || ((uchar)(c)) == 212 || ((uchar)(c)) == 213)
//#define os_dquote(c) ((c) == '"'  || ((uchar)(c)) == 210 || ((uchar)(c)) == 211)

/* os_qmatch is true if the delimiter characters match */
//#define os_qmatch(a, b)  ((os_squote(a) && os_squote(b)) || (os_dquote(a) && os_dquote(b)))

#define os_squote(c) ((c) == '\'')
#define os_dquote(c) ((c) == '"')
#define os_qmatch(a, b) ((a) == (b))

/*
 *  Unix systems have 'em, but we don't -- definitions for
 *  case-insensitive compares.
 */
int stricmp(const char *a, const char *b);
int strnicmp(const char *a, const char *b, size_t lim);
int memicmp(const void *a, const void *b, size_t lim);

/*
 *  Set plain ascii mode - does nothing on the mac
 */
#define os_plain()

/*
 *  Redefine NOTREACHED and NOTREACHEDV to always be real
 *  to cut down on warnings.
 */
#ifdef NOTREACHED
    #undef NOTREACHED
#endif
#define NOTREACHED  return
#ifdef NOTREACHEDV
    #undef NOTREACHEDV
#endif
#define NOTREACHEDV(t)  return ((t)0)


#endif /* MAC_OS */

#endif /* OSMAC_INCLUDED */

